package se.softstuff.nb.bundletracker;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import se.softstuff.nb.bundletracker.api.BundleRowNode;
import se.softstuff.nb.bundletracker.api.BundleEditorService;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import javax.swing.table.AbstractTableModel;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import se.softstuff.nb.bundletracker.option.BundleTrackerProperties;

/**
 *
 * @author stefan
 */
public class BundleRowNodeTableModel extends AbstractTableModel implements PropertyChangeListener{


    public enum COLUMN {
        BINARY, BUNDLE, KEY
    };
    
    private final List<BundleRowNode> rows;

    private final List<String> headers;
    
    private final BundleTrackerProperties options;

    public BundleRowNodeTableModel() {
        
        options = Lookup.getDefault().lookup(BundleTrackerProperties.class);
        
        options.addPropertyChangeListener(this);
                
        rows = new ArrayList<BundleRowNode>();
        headers = new ArrayList<String>();
        initHeaders();

    }
    
    private void initHeaders() {
        headers.clear();
        headers.add(NbBundle.getMessage(this.getClass(), "Table.column.header.binary")); //NOI18N
        headers.add(NbBundle.getMessage(this.getClass(), "Table.column.header.bundle")); //NOI18N
        headers.add(NbBundle.getMessage(this.getClass(), "Table.column.header.key")); //NOI18N
        for (Locale lang : options.getSelectedLanguages()) {
            headers.add(lang.getDisplayLanguage());
        }
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if(evt.getPropertyName().equals(BundleTrackerProperties.PROP_SELECTED_LANGUAGES)){
            
            initHeaders();
            fireTableStructureChanged();
        }
    }
    @Override
    public int getRowCount() {
        return rows.size();
    }

    @Override
    public int getColumnCount() {
        return COLUMN.values().length + options.getSelectedLanguages().size();
    }

    @Override
    public String getColumnName(int column) {
        if(column>=headers.size()){
            return ""; //NOI18N
        }
        return headers.get(column);
    }

    
    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        if (rowIndex >= rows.size() || columnIndex >= getColumnCount()) {
            return null;
        }
        BundleRowNode row = rows.get(rowIndex);
        switch (columnIndex) {
            case 0:
                return row.getBinary();
            case 1:
                return row.getBundlePackage();
            case 2:
                return row.getBundleKey();
            default:
                return row.getValue(getLanguage(columnIndex));
        }
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return String.class;
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        Iterator<? extends BundleEditorService> iterator = Lookup.getDefault().lookupAll(BundleEditorService.class).iterator();
        while(iterator.hasNext()){
            if(iterator.next().isEditable()){
                return true;
            }
        }
        return super.isCellEditable(rowIndex, columnIndex);
    }

    
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        BundleRowNode row = rows.get(rowIndex);
        Locale language = getLanguage(columnIndex);
        String value = (String)aValue;
        
        Iterator<? extends BundleEditorService> iterator = Lookup.getDefault().lookupAll(BundleEditorService.class).iterator();
        while(iterator.hasNext()){
            value = iterator.next().setValue(row, language, value);
        }
        super.setValueAt(value, rowIndex, columnIndex);
        fireTableCellUpdated(rowIndex, columnIndex);
    }

    
    
    private Locale getLanguage(int columnIndex) {
        return options.getSelectedLanguages().get(columnIndex - COLUMN.values().length);
    }
    
    
    public void setRows(Collection<? extends BundleRowNode> rows ){
        this.rows.clear();
        this.rows.addAll(rows);
        fireTableDataChanged();
    }
}
