package se.softstuff.nb.bundletracker;

import se.softstuff.nb.bundletracker.api.BundleRowNode;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.logging.Logger;
import javax.swing.RowFilter;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import org.openide.util.Lookup.Result;
import org.openide.util.LookupEvent;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
//import org.openide.util.ImageUtilities;
import org.netbeans.api.settings.ConvertAsProperties;
import org.openide.util.Lookup;
import org.openide.util.LookupListener;
import org.openide.util.NbPreferences;
import se.softstuff.nb.guicomponentmarker.api.DontMarkThisComponent;
import se.softstuff.nb.guicomponentmarker.api.HighlightedComponent;
import se.softstuff.nb.guicomponentmarker.api.TracerManager;

/**
 * Top component which displays something.
 */
@ConvertAsProperties(dtd = "-//se.softstuff.nb.bundletracker//BundleTracker//EN", //NOI18N
autostore = false)
public final class BundleTrackerTopComponent extends TopComponent implements LookupListener, DontMarkThisComponent {

    public static final String PROP_COMPONENTNAMEFILTER = "componentNameFilter"; //NOI18N

    public static final String PROP_REGEXP = "regexp"; //NOI18N

    public static final String PROP_REGEXP2 = "regexp2"; //NOI18N

    public static final String PREFERRED_ID = "BundleTrackerTopComponent"; //NOI18N

    private static BundleTrackerTopComponent instance;

    private String regexp;

    private String regexp2;

    private boolean componentNameFilter;
    private final TracerManager manager;

    private Result<HighlightedComponent> lookupResult;

    private BundleRowNodeTableModel model = new BundleRowNodeTableModel();

    private TableRowSorter<TableModel> sorter;

    private Map<String, BundleRowNode> memory = new HashMap<String, BundleRowNode>();

    public BundleTrackerTopComponent() {

        initComponents();
        setName(NbBundle.getMessage(BundleTrackerTopComponent.class, "CTL_BundleTrackerTopComponent")); //NOI18N
        setToolTipText(NbBundle.getMessage(BundleTrackerTopComponent.class, "HINT_BundleTrackerTopComponent")); //NOI18N

        bundelTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE); //NOI18N
        sorter = new TableRowSorter<TableModel>(model);
        bundelTable.setRowSorter(sorter);
        manager = (TracerManager) Lookup.getDefault().lookup(TracerManager.class);

        addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (evt.getPropertyName().equals(PROP_COMPONENTNAMEFILTER) || evt.getPropertyName().equals(PROP_REGEXP) || evt.getPropertyName().equals(PROP_REGEXP2)) {
                    sorter.setRowFilter(buildFilter());
                }
            }
        });
    }

    public String getRegexp() {
        return regexp;
    }

    public void setRegexp(String regexp) {
        String oldRegexp = this.regexp;
        this.regexp = regexp;
        firePropertyChange(PROP_REGEXP, oldRegexp, regexp);
    }

    public boolean isComponentNameFilter() {
        return componentNameFilter;
    }

    public void setComponentNameFilter(boolean componentNameFilter) {
        boolean oldComponentNameFilter = this.componentNameFilter;
        this.componentNameFilter = componentNameFilter;
        firePropertyChange(PROP_COMPONENTNAMEFILTER, oldComponentNameFilter, componentNameFilter);
    }

    public String getRegexp2() {
        return regexp2;
    }

    public void setRegexp2(String regexp2) {
        String oldRegexp2 = this.regexp2;
        this.regexp2 = regexp2;
        firePropertyChange(PROP_REGEXP2, oldRegexp2, regexp2);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        bindingGroup = new org.jdesktop.beansbinding.BindingGroup();

        jScrollPane1 = new javax.swing.JScrollPane();
        bundelTable = new javax.swing.JTable();
        jPanel1 = new javax.swing.JPanel();
        compNameCheckBox = new javax.swing.JCheckBox();
        langLabel = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        filterRegExpTextField = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        filterRegExpTextField2 = new javax.swing.JTextField();

        setName("Form"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        bundelTable.setAutoCreateRowSorter(true);
        bundelTable.setModel(model);
        bundelTable.setName("bundelTable"); // NOI18N
        jScrollPane1.setViewportView(bundelTable);

        jPanel1.setName("jPanel1"); // NOI18N

        org.openide.awt.Mnemonics.setLocalizedText(compNameCheckBox, NbBundle.getMessage(BundleTrackerTopComponent.class, "BundleTrackerTopComponent.compNameCheckBox.text")); // NOI18N
        compNameCheckBox.setName("compNameCheckBox"); // NOI18N

        org.jdesktop.beansbinding.Binding binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, this, org.jdesktop.beansbinding.ELProperty.create("${componentNameFilter}"), compNameCheckBox, org.jdesktop.beansbinding.BeanProperty.create("selected"));
        bindingGroup.addBinding(binding);

        org.openide.awt.Mnemonics.setLocalizedText(langLabel, NbBundle.getMessage(BundleTrackerTopComponent.class, "BundleTrackerTopComponent.langLabel.text", Locale.getDefault().getDisplayLanguage())); // NOI18N
        langLabel.setName("langLabel"); // NOI18N

        org.openide.awt.Mnemonics.setLocalizedText(jLabel2, NbBundle.getMessage(BundleTrackerTopComponent.class, "BundleTrackerTopComponent.jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jLabel1.setLabelFor(filterRegExpTextField);
        org.openide.awt.Mnemonics.setLocalizedText(jLabel1, NbBundle.getMessage(BundleTrackerTopComponent.class, "BundleTrackerTopComponent.jLabel1.text", Locale.getDefault().getDisplayLanguage())); // NOI18N
        jLabel1.setToolTipText(NbBundle.getMessage(BundleTrackerTopComponent.class, "BundleTrackerTopComponent.jLabel1.toolTipText", Locale.getDefault().getDisplayLanguage())); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        filterRegExpTextField.setName("filterRegExpTextField"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, this, org.jdesktop.beansbinding.ELProperty.create("${regexp}"), filterRegExpTextField, org.jdesktop.beansbinding.BeanProperty.create("text"));
        bindingGroup.addBinding(binding);

        jLabel3.setLabelFor(filterRegExpTextField2);
        org.openide.awt.Mnemonics.setLocalizedText(jLabel3, NbBundle.getMessage(BundleTrackerTopComponent.class, "BundleTrackerTopComponent.jLabel3.text", Locale.getDefault().getDisplayLanguage())); // NOI18N
        jLabel3.setToolTipText(NbBundle.getMessage(BundleTrackerTopComponent.class, "BundleTrackerTopComponent.jLabel3.toolTipText", Locale.getDefault().getDisplayLanguage())); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        filterRegExpTextField2.setName("filterRegExpTextField2"); // NOI18N

        binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, this, org.jdesktop.beansbinding.ELProperty.create("${regexp2}"), filterRegExpTextField2, org.jdesktop.beansbinding.BeanProperty.create("text"));
        bindingGroup.addBinding(binding);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 37, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(compNameCheckBox, javax.swing.GroupLayout.PREFERRED_SIZE, 121, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 45, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(filterRegExpTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 145, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 72, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(filterRegExpTextField2, javax.swing.GroupLayout.DEFAULT_SIZE, 145, Short.MAX_VALUE)
                .addGap(52, 52, 52)
                .addComponent(langLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 174, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(compNameCheckBox)
                    .addComponent(jLabel1)
                    .addComponent(langLabel)
                    .addComponent(filterRegExpTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3)
                    .addComponent(filterRegExpTextField2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 829, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 30, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 275, Short.MAX_VALUE))
        );

        bindingGroup.bind();
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTable bundelTable;
    private javax.swing.JCheckBox compNameCheckBox;
    private javax.swing.JTextField filterRegExpTextField;
    private javax.swing.JTextField filterRegExpTextField2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel langLabel;
    private org.jdesktop.beansbinding.BindingGroup bindingGroup;
    // End of variables declaration//GEN-END:variables

    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link #findInstance}.
     */
    public static synchronized BundleTrackerTopComponent getDefault() {
        if (instance == null) {
            instance = new BundleTrackerTopComponent();
        }
        return instance;
    }

    /**
     * Obtain the BundleTrackerTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized BundleTrackerTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            Logger.getLogger(BundleTrackerTopComponent.class.getName()).warning(
                    "Cannot find " + PREFERRED_ID + " component. It will not be located properly in the window system.");  //NOI18N
            return getDefault();
        }
        if (win instanceof BundleTrackerTopComponent) {
            return (BundleTrackerTopComponent) win;
        }
        Logger.getLogger(BundleTrackerTopComponent.class.getName()).warning(
                "There seem to be multiple components with the '" + PREFERRED_ID
                + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_NEVER;
    }

    @Override
    public void componentOpened() {

        lookupResult = manager.getLookup().lookupResult(HighlightedComponent.class);
        lookupResult.addLookupListener(this);
        compNameCheckBox.setSelected(NbPreferences.forModule(this.getClass()).getBoolean("compNameCheckBox.selected", true));

        reload();
    }

    @Override
    public void componentClosed() {
        NbPreferences.forModule(this.getClass()).putBoolean("compNameCheckBox.selected", compNameCheckBox.isSelected());
        lookupResult.removeLookupListener(this);
        lookupResult = null;
    }

    void writeProperties(java.util.Properties p) {
    }

    Object readProperties(java.util.Properties p) {
        if (instance == null) {
            instance = this;
        }
        instance.readPropertiesImpl(p);
        return instance;
    }

    private void readPropertiesImpl(java.util.Properties p) {
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }

    @Override
    public void resultChanged(LookupEvent ev) {
        reload();
    }

    private void reload() {
        Collection<? extends HighlightedComponent> allInstances = lookupResult.allInstances();
        memory.clear();

        List<BundleRowNode> rows = new ArrayList<BundleRowNode>();
        for (HighlightedComponent highlightedComponent : allInstances) {
            rows.addAll(createNodeFor(highlightedComponent.getComponent()));
        }
        sorter.setRowFilter(buildFilter());
        model.setRows(rows);
        bundelTable.addNotify();

    }

    private RowFilter<? super TableModel, ? super Integer> buildFilter() {
        List<RowFilter<? super TableModel, ? super Integer>> filters = new LinkedList<RowFilter<? super TableModel, ? super Integer>>();
        Collection<? extends HighlightedComponent> allInstances = lookupResult.allInstances();
        for (HighlightedComponent highlightedComponent : allInstances) {
            if (compNameCheckBox.isSelected()) {
                Component component = highlightedComponent.getComponent();
                String name = component.getName();
                if (name != null) {
                    filters.add(RowFilter.regexFilter(name));
                }
            }
        }
        if (!filterRegExpTextField.getText().isEmpty()) {
            filters.add(RowFilter.regexFilter(filterRegExpTextField.getText()));
        }
        if (!filterRegExpTextField2.getText().isEmpty()) {
            filters.add(RowFilter.regexFilter(filterRegExpTextField2.getText()));
        }
        return RowFilter.andFilter((Iterable) filters);
    }

    private List<BundleRowNode> createNodeFor(Component component) {

        List<BundleRowNode> nodes = new LinkedList<BundleRowNode>();
        if (component == null) {
            return nodes;
        }

        BundleRowNode node = null;
        Class clazz = component.getClass();
        String bundlePackage = clazz.getPackage().getName();
        if (!memory.containsKey(bundlePackage)) {

            try {
                ResourceBundle bundle = NbBundle.getBundle(clazz);
                node = new BundleRowNode(bundlePackage, null, clazz);
                memory.put(bundlePackage, node);

                Enumeration<String> keys = bundle.getKeys();
                while (keys.hasMoreElements()) {
                    String key = keys.nextElement();

                    nodes.add(new BundleRowNode(bundlePackage, key, clazz));
                }
            } catch (java.util.MissingResourceException ignore) {
            }
        }

        // do not go deeper the the first top component
        if (component instanceof TopComponent) {
            return nodes;
        }

        List<BundleRowNode> parent = createNodeFor(component.getParent());
        if (parent != null) {
            nodes.addAll(parent);
        }
        return nodes;
    }
}
