package se.softstuff.nb.bundletracker.api;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import org.openide.util.NbBundle;
import se.softstuff.nb.guicomponentmarker.api.MarkerUtilities;

/**
 * Represents the row model
 * @author stefan
 */
public class BundleRowNode {

    private final String bundlePackage;

    private final String bundleKey;

    private final String binary;

    private final Class markedComponent;
    
    private final String baseName; 

    /**
     * 
     * @param key bundle key
     * @param clazz the marked component class
     */
    public BundleRowNode(String bundlePackage, String bundleKey, Class markedComponent) {
        this.bundlePackage = bundlePackage;
        this.bundleKey = bundleKey;
        this.markedComponent = markedComponent;
        this.binary = MarkerUtilities.findBinaryFor(markedComponent);
        this.baseName = findName(markedComponent);
    }

    public String getBundlePackage() {
        return bundlePackage;
    }

    public String getBundleKey() {
        return bundleKey;
    }

    public String getValue(Locale language) {
        try {
            
            ResourceBundle bundle = NbBundle.getBundle(baseName, language, markedComponent.getClassLoader());
            if (bundle.getLocale().equals(language)) {
                String value = bundle.getString(bundleKey);
                return value;
            } else {
                return null;
            }
        } catch (MissingResourceException ex) {
            return null;
        }
    }

    @Override
    public String toString() {
        return bundlePackage;
    }

    /** Finds package name for given class */
    private static String findName(Class clazz) {
        String pref = clazz.getName();
        int last = pref.lastIndexOf('.');

        if (last >= 0) {
            pref = pref.substring(0, last + 1);

            return pref + "Bundle"; // NOI18N
        } else {
            // base package, search for bundle
            return "Bundle"; // NOI18N
        }
    }

    public String getBinary() {
        return binary;
    }
}
