package se.softstuff.nb.bundletracker.option;

import java.text.Collator;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import javax.swing.DefaultListModel;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * The options panel for selecting what languages to display in the rows.
 * Selected languages is persisted in the NetBeans user dir
 * @author stefan
 */
final class BundleTrackerOptionPanel extends javax.swing.JPanel {

    public static final String DELIMITER = ";"; // NOI18N

    public static final String PROP_SELECTED = "selected"; // NOI18N

    private final OptionOptionsPanelController controller;

    private DefaultListModel availableListModel = new DefaultListModel();

    private DefaultListModel selectedListModel = new DefaultListModel();

    BundleTrackerOptionPanel(OptionOptionsPanelController controller) {
        this.controller = controller;
        initComponents();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        languagePanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        selectedJList = new javax.swing.JList();
        addLangButton = new javax.swing.JButton();
        removeLangButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        availableJList = new javax.swing.JList();

        setName("Form"); // NOI18N

        languagePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(NbBundle.getMessage(BundleTrackerOptionPanel.class, "BundleTrackerOptionPanel.languagePanel.border.title", Locale.getDefault().getDisplayLanguage()))); // NOI18N
        languagePanel.setName("languagePanel"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        selectedJList.setModel(selectedListModel);
        selectedJList.setCellRenderer(new LanguageListRendrer());
        selectedJList.setName("selectedJList"); // NOI18N
        jScrollPane1.setViewportView(selectedJList);

        org.openide.awt.Mnemonics.setLocalizedText(addLangButton, NbBundle.getMessage(BundleTrackerOptionPanel.class, "BundleTrackerOptionPanel.addLangButton.text", Locale.getDefault().getDisplayLanguage())); // NOI18N
        addLangButton.setName("addLangButton"); // NOI18N
        addLangButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addLangButtonActionPerformed(evt);
            }
        });

        org.openide.awt.Mnemonics.setLocalizedText(removeLangButton, NbBundle.getMessage(BundleTrackerOptionPanel.class, "BundleTrackerOptionPanel.removeLangButton.text", Locale.getDefault().getDisplayLanguage())); // NOI18N
        removeLangButton.setName("removeLangButton"); // NOI18N
        removeLangButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                removeLangButtonActionPerformed(evt);
            }
        });

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        availableJList.setModel(availableListModel);
        availableJList.setCellRenderer(new LanguageListRendrer());
        availableJList.setName("availableJList"); // NOI18N
        jScrollPane2.setViewportView(availableJList);

        javax.swing.GroupLayout languagePanelLayout = new javax.swing.GroupLayout(languagePanel);
        languagePanel.setLayout(languagePanelLayout);
        languagePanelLayout.setHorizontalGroup(
            languagePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, languagePanelLayout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 118, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(languagePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(removeLangButton)
                    .addComponent(addLangButton))
                .addGap(14, 14, 14)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 118, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        languagePanelLayout.setVerticalGroup(
            languagePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, languagePanelLayout.createSequentialGroup()
                .addGap(23, 23, 23)
                .addComponent(addLangButton, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(removeLangButton)
                .addGap(88, 88, 88))
            .addGroup(languagePanelLayout.createSequentialGroup()
                .addGroup(languagePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING, 0, 0, Short.MAX_VALUE)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 105, Short.MAX_VALUE))
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(languagePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(languagePanel, javax.swing.GroupLayout.PREFERRED_SIZE, 137, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void addLangButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addLangButtonActionPerformed

        for (Locale sel : convert(availableJList.getSelectedValues())) {
            select(sel);
        }
        sortList(selectedListModel);
        controller.changed();
    }//GEN-LAST:event_addLangButtonActionPerformed

    private void removeLangButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_removeLangButtonActionPerformed
        for (Locale language : convert(selectedJList.getSelectedValues())) {
            availableListModel.addElement(language);
            selectedListModel.removeElement(language);
        }
        sortList(availableListModel);
        controller.changed();
    }//GEN-LAST:event_removeLangButtonActionPerformed

    public List<Locale> getAvailable() {
        return convert(availableListModel.toArray());
    }

    public List<Locale> getSelected() {
        return convert(selectedListModel.toArray());
    }

    List<Locale> loadSelectedLanguages() {
        return Lookup.getDefault().lookup(BundleTrackerProperties.class).getSelectedLanguages();
    }

    void load() {
        availableListModel.clear();
        selectedListModel.clear();

        for (Locale language : Locale.getAvailableLocales()) {
            availableListModel.addElement(language);
        }

        for (Locale selected : loadSelectedLanguages()) {
            select(selected);
        }

        sortList(availableListModel);
        sortList(selectedListModel);
    }

    void store() {
        Lookup.getDefault().lookup(BundleTrackerProperties.class).setSelecteLanguages(getSelected());
    }

    boolean valid() {
        return true;
    }

    private void select(Locale language) {
        selectedListModel.addElement(language);
        availableListModel.removeElement(language);
    }

    private static List<Locale> convert(Object[] values) {

        Locale[] languages = Arrays.copyOf(values, values.length, Locale[].class);
        return Arrays.asList(languages);
    }

    private void sortList(DefaultListModel model) {
        int numItems = model.getSize();
        Object[] pos = new Object[numItems];

        for (int i = 0; i < numItems; i++) {
            pos[i] = model.getElementAt(i);
        }
        sortArray(Collator.getInstance(), pos);
        for (int i = 0; i < numItems; i++) {
            model.setElementAt(pos[i], i);

        }
    }

    public static void sortArray(Collator collator, Object[] arrToSort) {
        Object tmp;
        if (arrToSort.length == 1) {
            return;
        }
        for (int i = 0; i < arrToSort.length; i++) {
            for (int j = i + 1; j < arrToSort.length; j++) {
                Locale a = (Locale) arrToSort[i];
                Locale b = (Locale) arrToSort[j];
                if (collator.compare(a.getDisplayName(Locale.ENGLISH), b.getDisplayName(Locale.ENGLISH)) > 0) {
                    tmp = arrToSort[i];
                    arrToSort[i] = arrToSort[j];
                    arrToSort[j] = tmp;
                }
            }
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addLangButton;
    private javax.swing.JList availableJList;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JPanel languagePanel;
    private javax.swing.JButton removeLangButton;
    private javax.swing.JList selectedJList;
    // End of variables declaration//GEN-END:variables

}
