package se.softstuff.nb.guicomponentmarker.api;

import java.awt.Color;
import java.awt.Component;
import java.util.ListIterator;
import java.util.Random;
import org.openide.windows.TopComponent;

/**
 * The keeper of the component and color 
 * @author stefan
 */
public class HighlightedComponent {
    
    
    private static final Random random = new Random();
    
    public static HighlightedComponent create(Component component, ListIterator<Color> colorStack){
        if(component==null){
            return null;
        }
        Color color;
        if(!colorStack.hasNext()){
            color = new Color(random.nextInt());
            colorStack.add(color);
        } else {
            color = colorStack.next();
        }
        HighlightedComponent highlight = new HighlightedComponent(component, color);
        
        if(!(component instanceof TopComponent)) {
            highlight.setParent(create(component.getParent(), colorStack));
        }
        return highlight;
    }
    
    
    private final Color color;
    
    private final Component component;
    
    private HighlightedComponent parent;

    private HighlightedComponent(Component component, Color color) {
        this.color = color;
        this.component = component;
    }
    
    
    public Color getColor() {
        return color;
    }

    public Component getComponent() {
        return component;
    }
    
    /**
     * the HighlightedComponent object for component.getParent object
     * @return the keeper of the color
     */
    public HighlightedComponent getParent() {
        return parent;
    }

    private void setParent(HighlightedComponent parent) {
        this.parent = parent;
    }
    
}
